@@ -1,5 +1,5 @@
 /* This file is part of gacopyz.
-   Copyright (C) 2006-2023 Sergey Poznyakoff
+   Copyright (C) 2006-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -182,7 +182,6 @@ gacopyz_parse_connection(const char *cstr,
 	return MI_SUCCESS;
 }
 
-
 static int
 parse_connection(gacopyz_conn_t conn,
 		 const char *cstr,
@@ -765,7 +764,7 @@ ctx_write(SMFICTX *ctx, const char *buf, size_t size)
 
 union header {
 	struct {
-		gacopyz_uint32_t size;
+		uint32_t size;
 		unsigned char cmd;
 	} hdr;
 	char buf[5];
@@ -839,10 +838,10 @@ convert_sfsistat(sfsistat stat)
 	return SMFIR_TEMPFAIL; /* Just in case */
 }
 
-#define OPTLEN 3*sizeof(gacopyz_uint32_t)
+#define OPTLEN 3*sizeof(uint32_t)
 
 static int
-make_optneg_buf(SMFICTX *ctx, gacopyz_uint32_t *vbuf,
+make_optneg_buf(SMFICTX *ctx, uint32_t *vbuf,
 		size_t *psize, char **pbuf)
 {
 	char *buf;
@@ -852,7 +851,7 @@ make_optneg_buf(SMFICTX *ctx, gacopyz_uint32_t *vbuf,
 	for (i = 0; i < gacopyz_stage_max; i++) {
 		if (ctx->req_macros[i])
 			bufsize += strlen(ctx->req_macros[i]) + 1 +
-				     sizeof(gacopyz_uint32_t);
+				     sizeof(uint32_t);
 	}
 
 	if (bufsize == 0)
@@ -862,7 +861,7 @@ make_optneg_buf(SMFICTX *ctx, gacopyz_uint32_t *vbuf,
 		buf = malloc(bufsize);
 		if (!buf)
 			return MI_FAILURE;
-		vbuf = (gacopyz_uint32_t*)buf;
+		vbuf = (uint32_t*)buf;
 	}
 	vbuf[0] = htonl(ctx->version);
 	vbuf[1] = htonl(ctx->aflags);
@@ -876,7 +875,7 @@ make_optneg_buf(SMFICTX *ctx, gacopyz_uint32_t *vbuf,
 		buf += OPTLEN;
 		for (i = 0; i < gacopyz_stage_max; i++) {
 			if (ctx->req_macros[i]) {
-				gacopyz_uint32_t v;
+				uint32_t v;
 				size_t len;
 
 				v = htonl(i);
@@ -899,7 +898,7 @@ send_reply(SMFICTX *ctx, unsigned char cmd)
 	char *buf = NULL;
 	size_t bufsize = 0;
 	char *alloc_mem = NULL;
-	gacopyz_uint32_t v[3];
+	uint32_t v[3];
 	unsigned long nrmask = state_nr_mask[ctx->state];
 
 	if (nrmask && (ctx->pflags & nrmask) && cmd != _GACOPYZ_R_NOREPLY) {
@@ -1005,7 +1004,7 @@ union state_arg {
 		size_t len;
 	} string;
 	char *strings[2];
-	gacopyz_uint32_t ints[3];
+	uint32_t ints[3];
 	struct {
 		char cmd;
 		char **v;
@@ -1319,7 +1318,7 @@ shan_mail(SMFICTX *ctx, union state_arg *arg, unsigned char *cmd)
 static state_ret_type
 shan_optneg(SMFICTX *ctx, union state_arg *arg, unsigned char *cmd)
 {
-	gacopyz_uint32_t val;
+	uint32_t val;
 	unsigned long mta_aflags, mta_pflags;
 
 	val = arg->ints[0];
@@ -1562,9 +1561,9 @@ parse_state_arg(union state_arg *arg, enum state_arg_type type,
 		if (size < GACOPYZ_OPTLEN) 
 			return MI_FAILURE;
 				   
-		arg->ints[0] = ntohl(((gacopyz_uint32_t*)buffer)[0]);
-		arg->ints[1] = ntohl(((gacopyz_uint32_t*)buffer)[1]);
-		arg->ints[2] = ntohl(((gacopyz_uint32_t*)buffer)[2]);
+		arg->ints[0] = ntohl(((uint32_t*)buffer)[0]);
+		arg->ints[1] = ntohl(((uint32_t*)buffer)[1]);
+		arg->ints[2] = ntohl(((uint32_t*)buffer)[2]);
 		return MI_SUCCESS;
 	}
 	
@@ -2024,15 +2023,15 @@ gacopyz_header_command0(SMFICTX *ctx, unsigned char cmd,
 	lenv = strlen(headerv) + 1;
 	bufsize = lenf + lenv;
 	if (idx >= 0)
-		bufsize += sizeof(gacopyz_uint32_t);
+		bufsize += sizeof(uint32_t);
 	buf = malloc(sizeof hptr->buf + bufsize);
 	if (!buf)
 		return MI_FAILURE;
 	hptr = (union header *) buf;
 	cptr = buf + sizeof hptr->buf;
 	if (idx >= 0) {
-		*(gacopyz_uint32_t*)cptr = htonl(idx);
-		cptr += sizeof(gacopyz_uint32_t);
+		*(uint32_t*)cptr = htonl(idx);
+		cptr += sizeof(uint32_t);
 	}
 	memcpy(cptr, headerf, lenf);
 	cptr += lenf;
