@@ -1,5 +1,5 @@
 /* This file is part of gacopyz.
-   Copyright (C) 2007-2023 Sergey Poznyakoff
+   Copyright (C) 2007-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1075,23 +1075,23 @@ parse_macros(gacopyz_srv_t srv, enum gacopyz_stage stage, char *buf)
 static void
 read_macros(gacopyz_srv_t srv)
 {
-	char *buf = srv->buf + 3 * sizeof(gacopyz_uint32_t);
-	size_t size = srv->bufsize - 3 * sizeof(gacopyz_uint32_t);
+	char *buf = srv->buf + 3 * sizeof(uint32_t);
+	size_t size = srv->bufsize - 3 * sizeof(uint32_t);
 		
-	while (size > sizeof(gacopyz_uint32_t)) {
-		gacopyz_uint32_t v;
+	while (size > sizeof(uint32_t)) {
+		uint32_t v;
 		unsigned n;
 		size_t len;
 			
-		memcpy(&v, buf, sizeof(gacopyz_uint32_t));
+		memcpy(&v, buf, sizeof(uint32_t));
 		n = ntohl(v);
 		if (n >= gacopyz_stage_max) {
 			gacopyz_io_log(&srv->iod, SMI_LOG_ERR,
 				       _("received invalid stage number"));
 			break;
 		}
-		buf += sizeof(gacopyz_uint32_t);
-		size -= sizeof(gacopyz_uint32_t);
+		buf += sizeof(uint32_t);
+		size -= sizeof(uint32_t);
 		if (size == 0) 
 			break;
 		len = strlen(buf) + 1;
@@ -1112,7 +1112,7 @@ read_macros(gacopyz_srv_t srv)
 int
 gacopyz_srv_negotiate(gacopyz_srv_t srv)
 {
-	gacopyz_uint32_t ibuf[3];
+	uint32_t ibuf[3];
 	size_t size;
 	unsigned char cmd;
 	unsigned long milter_version, milter_acts, milter_proto;
@@ -1158,9 +1158,9 @@ gacopyz_srv_negotiate(gacopyz_srv_t srv)
 		return MI_FAILURE;
 	}
 
-	milter_version = ntohl(((gacopyz_uint32_t*)srv->buf)[0]);
-	milter_acts    = ntohl(((gacopyz_uint32_t*)srv->buf)[1]);
-	milter_proto   = ntohl(((gacopyz_uint32_t*)srv->buf)[2]);
+	milter_version = ntohl(((uint32_t*)srv->buf)[0]);
+	milter_acts    = ntohl(((uint32_t*)srv->buf)[1]);
+	milter_proto   = ntohl(((uint32_t*)srv->buf)[2]);
 
 	/* Check version */
 	if (milter_version == 1 || milter_version > SMFI_VERSION
@@ -1190,7 +1190,7 @@ gacopyz_srv_negotiate(gacopyz_srv_t srv)
 	}
 
 	if (milter_version > 3
-	    && srv->bufsize > 3 * sizeof(gacopyz_uint32_t))
+	    && srv->bufsize > 3 * sizeof(uint32_t))
 		read_macros(srv);
 	
 	srv->version  = milter_version;
