@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2023 Sergey Poznyakoff
+   Copyright (C) 2005-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <mailutils/wordsplit.h>
 #include <mailutils/assoc.h>
 
+#include <mflib/exceptions.h>
 #include "libmf.h"
 #include "dns.h"
 
@@ -101,7 +102,7 @@ dnsbase_file_init(char const *filename)
 		mu_off_t sz;
 		int rc;
 		char *cfg;
-		
+
 		rc = mu_file_stream_create(&str, filename, MU_STREAM_READ);
 		if (rc) {
 			mu_diag_funcall(MU_DIAG_ERROR, "mu_file_stream_create",
@@ -121,7 +122,7 @@ dnsbase_file_init(char const *filename)
 			mu_stream_destroy(&str);
 			return;
 		}
-		
+
 		cfg = mu_alloc(sz + 1);
 
 		rc = mu_stream_read(str, cfg, sz, NULL);
@@ -145,12 +146,14 @@ get_state(void)
 	return state;
 }
 
-static inline size_t
+size_t
 dns_reply_elsize(struct dns_reply *reply)
 {
 	switch (reply->type) {
 	case dns_reply_ip:
 		return sizeof(reply->data.ip[0]);
+	case dns_reply_ip6:
+		return sizeof(reply->data.ip6[0]);
 	case dns_reply_str:
 		return sizeof(reply->data.str[0]);
 	}
@@ -172,18 +175,28 @@ dns_reply_init(struct dns_reply *reply, dns_reply_type type, size_t count)
 void
 dns_reply_ip_push(struct dns_reply *reply, void *item)
 {
-	if (reply->count == reply->maxcount) 
+	if (reply->count == reply->maxcount)
 		reply->data.ip = mu_2nrealloc(reply->data.ip,
 					      &reply->maxcount,
 					      sizeof(reply->data.ip[0]));
-	reply->data.ip[reply->count++] = *(GACOPYZ_UINT32_T*)item;
+	reply->data.ip[reply->count++] = *(struct in_addr*)item;
+}
+
+void
+dns_reply_ip6_push(struct dns_reply *reply, void *item)
+{
+	if (reply->count == reply->maxcount)
+		reply->data.ip6 = mu_2nrealloc(reply->data.ip6,
+					       &reply->maxcount,
+					       sizeof(reply->data.ip6[0]));
+	reply->data.ip6[reply->count++] = *(struct in6_addr *)item;
 }
 
 void
 dns_reply_str_push(struct dns_reply *reply, void *item)
 {
-	if (reply->count == reply->maxcount) 
-		reply->data.str = mu_2nrealloc(reply->data.ip,
+	if (reply->count == reply->maxcount)
+		reply->data.str = mu_2nrealloc(reply->data.str,
 					       &reply->maxcount,
 					       sizeof(reply->data.str[0]));
 	reply->data.str[reply->count++] = item;
@@ -196,6 +209,9 @@ dns_reply_push(struct dns_reply *reply, void *item)
 	case dns_reply_ip:
 		dns_reply_ip_push(reply, item);
 		break;
+	case dns_reply_ip6:
+		dns_reply_ip6_push(reply, item);
+		break;
 	case dns_reply_str:
 		dns_reply_str_push(reply, item);
 		break;
@@ -208,7 +224,7 @@ void
 dns_reply_free(struct dns_reply *reply)
 {
 	int i;
-	
+
 	switch (reply->type) {
 	case dns_reply_str:
 		for (i = 0; i < reply->count; i++)
@@ -218,29 +234,57 @@ dns_reply_free(struct dns_reply *reply)
 	case dns_reply_ip:
 		free(reply->data.ip);
 		break;
+	case dns_reply_ip6:
+		free(reply->data.ip6);
+		break;
 	}
-}	
+}
 
 int
 dns_str_is_ipv4(const char *addr)
 {
-        int dot_count;
-        int digit_count;
+	int dot_count;
+	int digit_count;
 
-        dot_count = 0;
-        digit_count = 0;
-        while (*addr != 0) {
-                if (*addr == '.') {
-                        if (++dot_count > 4)
-                                return 0;
-                        digit_count = 0;
-                } else if (!(isdigit(*addr) && ++digit_count <= 3)) {
-                        return 0;
-                }
-                addr++;
-        }
-	
-        return dot_count == 3;
+	dot_count = 0;
+	digit_count = 0;
+	while (*addr != 0) {
+		if (*addr == '.') {
+			if (++dot_count > 4)
+				return 0;
+			digit_count = 0;
+		} else if (!(isdigit(*addr) && ++digit_count <= 3)) {
+			return 0;
+		}
+		addr++;
+	}
+
+	return dot_count == 3;
+}
+
+int
+dns_str_is_ipv6(const char *addr)
+{
+	int col_count = 0; /* Number of colons */
+	int dcol = 0;      /* Did we encounter a double-colon? */
+	int dig_count = 0; /* Number of digits in the last group */
+
+	for (; *addr; addr++) {
+		if (!isascii(*addr))
+			return 0;
+		else if (isxdigit(*addr)) {
+			if (++dig_count > 4)
+				return 0;
+		} else if (*addr == ':') {
+			if (col_count && dig_count == 0 && ++dcol > 1)
+				return 0;
+			if (++col_count > 7)
+				return 0;
+			dig_count = 0;
+		} else
+			return 0;
+	}
+	return col_count == 7 || dcol;
 }
 
 static int
@@ -266,9 +310,9 @@ errno_to_dns_status(int e)
    Values are increased by 1 to be able to tell whether the entry is
    initialized or not. */
 int adns_to_dns_tab[] = {
-#define STAT(s) ((s)+1)	
+#define STAT(s) ((s)+1)
 	[adns_s_ok]                  = STAT(dns_success),
-	
+
 	[adns_s_nomemory]            = STAT(dns_failure),
 	[adns_s_unknownrrtype]       = STAT(dns_failure),
 	[adns_s_systemfail]          = STAT(dns_failure),
@@ -293,16 +337,16 @@ int adns_to_dns_tab[] = {
 	[adns_s_answerdomaininvalid] = STAT(dns_not_found),
 	[adns_s_answerdomaintoolong] = STAT(dns_not_found),
 	[adns_s_invaliddata]         = STAT(dns_not_found),
- 
+
 	/* permanent problems with the query */
 	[adns_s_querydomainwrong]    = STAT(dns_failure),
 	[adns_s_querydomaininvalid]  = STAT(dns_failure),
 	[adns_s_querydomaintoolong]  = STAT(dns_failure),
- 
+
 	/* permanent errors */
 	[adns_s_nxdomain]            = STAT(dns_not_found),
 	[adns_s_nodata]              = STAT(dns_not_found),
-#undef STAT	
+#undef STAT
 };
 
 /* Convert ADNS status code E to DNS status. */
@@ -442,7 +486,7 @@ dns_query(const char *name, adns_rrtype type, adns_answer **ans_ret)
 	    && dns_max_cname_chain > 1) {
 		struct cname_record_list cname_rec = CNAME_RECORD_LIST_INITIALIZER;
 
-                /* Record the queried name, first. */
+		/* Record the queried name, first. */
 		cname_install(&cname_rec, name);
 
 		/* Follow the CNAME chain. */
@@ -493,8 +537,26 @@ dns_query(const char *name, adns_rrtype type, adns_answer **ans_ret)
 	return rc;
 }
 
+typedef dns_status (*dns_lookup_fn)(const char *, struct dns_reply *);
+
+static dns_lookup_fn
+lookup_function(int resolve_family)
+{
+	switch (resolve_family) {
+	case resolve_ip4:
+		return a_lookup;
+		break;
+	case resolve_ip6:
+		return aaaa_lookup;
+		break;
+	default:
+		break;
+	}
+	return NULL;
+}
+
 dns_status
-soa_check(const char *name, int ip, struct dns_reply *reply)
+soa_check(const char *name, int resolve_family, struct dns_reply *reply)
 {
 	dns_status status = dns_failure;
 	int rc;
@@ -505,8 +567,11 @@ soa_check(const char *name, int ip, struct dns_reply *reply)
 		return errno_to_dns_status(rc);
 	status = adns_to_dns_status(ans->status);
 	if (status == dns_success) {
-		if (ip) {
-			status = a_lookup(ans->rrs.soa->mname, reply);
+		if (resolve_family != resolve_none) {
+			dns_lookup_fn lookup = lookup_function(resolve_family);
+			if (lookup == NULL)
+				return dns_failure;
+			status = lookup(ans->rrs.soa->mname, reply);
 		} else {
 			dns_reply_init(reply, dns_reply_str, 1);
 			reply->data.str[0] = mu_strdup (ans->rrs.soa->mname);
@@ -517,33 +582,44 @@ soa_check(const char *name, int ip, struct dns_reply *reply)
 }
 
 static dns_status
-dns_reply_resolve(struct dns_reply *reply)
+dns_reply_resolve(struct dns_reply *reply, int family)
 {
 	size_t i;
 	struct dns_reply res;
-	
-	dns_reply_init(&res, dns_reply_ip, 0);
+	dns_lookup_fn lookup;
+
+	if (family == resolve_none)
+		return dns_success;
+
+	lookup = lookup_function(family);
+	if (lookup == NULL)
+		return dns_failure;
+
+	dns_reply_init(&res, (family == resolve_ip4) ? dns_reply_ip : dns_reply_ip6, 0);
 	for (i = 0; i < reply->count; i++) {
 		struct dns_reply r;
-		dns_status stat = a_lookup(reply->data.str[i], &r);
+		dns_status stat = lookup(reply->data.str[i], &r);
 		if (stat == dns_success) {
 			size_t n;
 			for (n = 0; n < r.count; n++) {
-				dns_reply_push(&res, &r.data.ip[n]);
+				dns_reply_push(&res,
+					       (family == resolve_ip4)
+					       ? (void*) &r.data.ip[n]
+					       : (void*) &r.data.ip6[n]);
 			}
 			dns_reply_free(&r);
 		}
 	}
 	dns_reply_free(reply);
 	*reply = res;
-	if (res.count == 0) 
+	if (res.count == 0)
 		return dns_not_found;
 	return dns_success;
 }
 
 /* Return MX records for the given HOST. */
 dns_status
-mx_lookup(const char *host, int resolve, struct dns_reply *reply)
+mx_lookup(const char *host, int resolve_family, struct dns_reply *reply)
 {
 	dns_status status = dns_failure;
 	int rc;
@@ -556,78 +632,157 @@ mx_lookup(const char *host, int resolve, struct dns_reply *reply)
 	status = adns_to_dns_status(ans->status);
 	if (status != dns_success)
 		return status;
-	
+
 	dns_reply_init(reply, dns_reply_str, ans->nrrs);
 	for (i = 0; i < ans->nrrs; i++)
 		reply->data.str[i] = mu_strdup(ans->rrs.inthostaddr[i].ha.host);
 	free(ans);
 
-	if (resolve)
-		status = dns_reply_resolve(reply);
-	
+	status = dns_reply_resolve(reply, resolve_family);
+
 	return status;
 }
 
-typedef char IPBUF[3*4+3+1];
+static int
+cidr_reverse_ip4(char *buf, size_t bufsize, struct mu_cidr const *cidr)
+{
+	int i, n;
+	char *start = buf;
 
-int
-dns_reverse_ipstr(const char *ipstr, char *revipstr)
+	for (i = cidr->len - 1; i >= 0; i--) {
+		unsigned c = cidr->address[i];
+		n = snprintf(buf, bufsize, "%u", c);
+		if (n < 0 || n >= bufsize)
+			return -1;
+		buf += n;
+		bufsize -= n;
+		*buf++ = '.';
+		bufsize--;
+	}
+	return buf - start;
+}
+
+static int
+cidr_reverse_ip6(char *buf, size_t bufsize, struct mu_cidr const *cidr)
 {
 	int i;
-	const char *p;
-	char *q;
+	char *start = buf;
+	static char xdig[] = "0123456789abcdef";
 
-	q = revipstr + strlen(ipstr);
-	*q = 0;
-	for (i = 0, p = ipstr; *p && i < 4; i++) {
-		int len;
-			
-		for (len = 0; p[len] && p[len] != '.'; len++)
-			;
-		q -= len;
-		memcpy(q, p, len);
-		if (q > revipstr)
-			*--q = '.';
-		p += len;
-		if (*p == '.')
-			p++;
+	if (bufsize < IPV6_DOTTED_BUFSIZE)
+		return -1;
+	for (i = cidr->len - 1; i >= 0; i--) {
+		unsigned c = cidr->address[i];
+		*buf++ = xdig[c & 0xf];
+		*buf++ = '.';
+		*buf++ = xdig[(c >> 4) & 0xf];
+		*buf++ = '.';
 	}
-	
-	return *p || i != 4;
+	return buf - start;
+}
+
+static int
+cidr_reverse(struct mu_cidr const *cidr, char *buf, size_t bufsize)
+{
+	size_t n;
+	int (*rev)(char *, size_t, struct mu_cidr const *);
+
+	switch (cidr->family) {
+	case AF_INET:
+		n = IPV4_DOTTED_BUFSIZE;
+		rev = cidr_reverse_ip4;
+		break;
+
+	case AF_INET6:
+		n = IPV6_DOTTED_BUFSIZE;
+		rev = cidr_reverse_ip6;
+		break;
+
+	default:
+		return -1;
+	}
+
+	if (n > bufsize)
+		return -1;
+
+	return rev(buf, bufsize, cidr);
+}
+
+int
+cidr_to_arpa(struct mu_cidr const *cidr, char *buf, size_t bufsize)
+{
+	size_t n;
+	char const *domain;
+
+	switch (cidr->family) {
+	case AF_INET:
+		domain = IPV4_INADDR_DOMAIN;
+		break;
+
+	case AF_INET6:
+		domain = IPV6_INADDR_DOMAIN;
+		break;
+
+	default:
+		return -1;
+	}
+
+	n = cidr_reverse(cidr, buf, bufsize);
+	buf += n;
+	bufsize -= n;
+
+	if (strlen(domain) >= bufsize)
+		return -1;
+	strcpy(buf, domain);
+	return 0;
+}
+
+int
+dns_reverse_ipstr(const char *ipstr, char *buf, size_t bufsize)
+{
+	struct mu_cidr cidr;
+	if (mu_cidr_from_string(&cidr, ipstr))
+		return -1;
+	return cidr_reverse(&cidr, buf, bufsize);
 }
 
 dns_status
-dns_resolve_ipstr(const char *ipstr, const char *domain, char **hbuf)
+dns_reverse_name(const char *ipstr, char *buf, size_t bufsize)
+{
+	struct mu_cidr cidr;
+	if (mu_cidr_from_string(&cidr, ipstr))
+		return dns_failure;
+	if (cidr_to_arpa(&cidr, buf, bufsize))
+		return dns_failure;
+	return dns_success;
+}
+
+dns_status
+dns_resolve_ipstr(const char *ipstr, struct dns_reply *reply)
 {
 	dns_status status = dns_failure;
 	int rc;
 	adns_answer *ans;
-	char *name;
-	adns_rrtype type;
-	
-	if (!domain || strcasecmp(domain, "in-addr.arpa") == 0) {
-		IPBUF ipbuf;
-		if (!dns_str_is_ipv4(ipstr))
-			return dns_failure;
-                if (dns_reverse_ipstr(ipstr, ipbuf))
-                        return dns_failure;
-		mu_asprintf(&name, "%s.in-addr.arpa", ipbuf);
-		type = adns_r_ptr_raw;
-        } else {
-		mu_asprintf(&name, "%s.%s", ipstr, domain);
-		type = adns_r_a;
-        }
+	struct mu_cidr cidr;
+	char buf[IPMAX_INADDR_BUFSIZE];
+	size_t buflen = sizeof(buf);
 
-	rc = dns_query(name, type, &ans);
-	free(name);
+	if (mu_cidr_from_string(&cidr, ipstr))
+		return dns_failure;
+
+	if (cidr_to_arpa(&cidr, buf, buflen))
+		return dns_failure;
+
+	rc = dns_query(buf, adns_r_ptr_raw, &ans);
 	if (rc)
 		return errno_to_dns_status(rc);
 	status = adns_to_dns_status(ans->status);
+
 	if (status == dns_success) {
-		if (ans->type == adns_r_ptr_raw) {
-			*hbuf = mu_strdup(ans->rrs.str[0]);
-		} else {
-			*hbuf = mu_strdup(inet_ntoa(ans->rrs.inaddr[0]));
+		int i;
+		dns_reply_init(reply, dns_reply_str, ans->nrrs);
+		for (i = 0; i < ans->nrrs; i++) {
+			reply->data.str[i] = mu_strdup(ans->rrs.str[i]);
 		}
 	}
 	free(ans);
@@ -635,22 +790,51 @@ dns_resolve_ipstr(const char *ipstr, const char *domain, char **hbuf)
 }
 
 dns_status
-dns_resolve_hostname(const char *host, char **ipbuf)
+dns_resolve_hostname(const char *host, int family, char **ipbuf)
 {
 	dns_status status = dns_failure;
 	int rc;
 	adns_answer *ans;
-	
-	rc = dns_query(host, adns_r_a, &ans);
+	adns_rrtype type;
+
+	switch (family) {
+	case resolve_ip4:
+		type = adns_r_a;
+		break;
+
+	case resolve_ip6:
+		type = adns_r_aaaa;
+		break;
+
+	default:
+		return dns_failure;
+	}
+
+	rc = dns_query(host, type, &ans);
 	if (rc)
 		return errno_to_dns_status(rc);
 	status = adns_to_dns_status(ans->status);
-	if (status == dns_success)
-		*ipbuf = mu_strdup(inet_ntoa(ans->rrs.inaddr[0]));
+	if (status == dns_success) {
+		char ipstr[IPMAX_DOTTED_BUFSIZE];
+		switch (type) {
+		case adns_r_a:
+			inet_ntop(AF_INET, &ans->rrs.inaddr[0], ipstr,
+				  sizeof(ipstr));
+			break;
+		case adns_r_aaaa:
+			inet_ntop(AF_INET6, &ans->rrs.in6addr[0], ipstr,
+				  sizeof(ipstr));
+			break;
+		default:
+			status = dns_failure;
+		}
+		if (status == dns_success)
+			*ipbuf = mu_strdup(ipstr);
+	}
 	free(ans);
 	return status;
 }
-	
+
 
 dns_status
 a_lookup(const char *host, struct dns_reply *reply)
@@ -658,7 +842,7 @@ a_lookup(const char *host, struct dns_reply *reply)
 	dns_status status = dns_failure;
 	int rc;
 	adns_answer *ans;
-	
+
 	rc = dns_query(host, adns_r_a, &ans);
 	if (rc)
 		return errno_to_dns_status(rc);
@@ -667,28 +851,40 @@ a_lookup(const char *host, struct dns_reply *reply)
 		int i;
 		dns_reply_init(reply, dns_reply_ip, ans->nrrs);
 		for (i = 0; i < ans->nrrs; i++)
-			reply->data.ip[i] = ans->rrs.inaddr[i].s_addr;
+			reply->data.ip[i] = ans->rrs.inaddr[i];
 	}
 	free(ans);
 	return status;
 }
 
 dns_status
-ptr_lookup(struct in_addr ip, struct dns_reply *reply)
+aaaa_lookup(const char *host, struct dns_reply *reply)
 {
 	dns_status status = dns_failure;
 	int rc;
 	adns_answer *ans;
-	char *name;
 
-	ip.s_addr = ntohl(ip.s_addr);
-	mu_asprintf(&name, "%d.%d.%d.%d.in-addr.arpa",
-		    ip.s_addr & 0xff,
-		    (ip.s_addr >> 8) & 0xff,
-		    (ip.s_addr >> 16) & 0xff,
-		    (ip.s_addr >> 24) & 0xff);
-	rc = dns_query(name, adns_r_ptr_raw, &ans);
-	free(name);
+	rc = dns_query(host, adns_r_aaaa, &ans);
+	if (rc)
+		return errno_to_dns_status(rc);
+	status = adns_to_dns_status(ans->status);
+	if (status == dns_success) {
+		int i;
+		dns_reply_init(reply, dns_reply_ip6, ans->nrrs);
+		for (i = 0; i < ans->nrrs; i++)
+			reply->data.ip6[i] = ans->rrs.in6addr[i];
+	}
+	return status;
+}
+
+dns_status
+ptr_lookup(const char *host, struct dns_reply *reply)
+{
+	dns_status status = dns_failure;
+	int rc;
+	adns_answer *ans;
+
+	rc = dns_query(host, adns_r_ptr_raw, &ans);
 	if (rc)
 		return errno_to_dns_status(rc);
 	status = adns_to_dns_status(ans->status);
@@ -750,9 +946,9 @@ spf_lookup(const char *domain, char **rec)
 	if (status == dns_success) {
 		int i;
 		int found = -1;
-		
+
 		status = dns_not_found;
-		
+
 		for (i = 0; i < reply.count; i++) {
 			if (mu_c_strncasecmp(reply.data.str[i],
 					     VSPF1_STR, VSPF1_LEN) == 0
@@ -771,7 +967,7 @@ spf_lookup(const char *domain, char **rec)
 
 		if (status == dns_success)
 			*rec = mu_strdup(reply.data.str[found]);
-				
+
 		dns_reply_free(&reply);
 	}
 	return status;
@@ -783,7 +979,7 @@ dkim_lookup(const char *domain, const char *sel, char ***retval)
 	dns_status status;
 	struct dns_reply reply;
 	char *dk;
-	
+
 	if (mu_asprintf(&dk, "%s._domainkey.%s", sel, domain))
 		mu_alloc_die();
 	status = txt_lookup(dk, &reply);
@@ -801,34 +997,63 @@ dkim_lookup(const char *domain, const char *sel, char ***retval)
 	return status;
 }
 
+static int
+ipaddr_eq(struct dns_reply *reply, int i, void *addr)
+{
+	void *ip;
+
+	if (reply->type == dns_reply_ip) {
+		ip = &reply->data.ip[i];
+	} else {
+		ip = &reply->data.ip6[i];
+	}
+	return memcmp(ip, addr, dns_reply_elsize(reply));
+}
 /* rfc4408, chapter 5.5 */
 dns_status
 ptr_validate(const char *ipstr, struct dns_reply *reply)
 {
-	struct in_addr ip;
 	size_t i;
+	union {
+		struct in_addr ip4;
+		struct in6_addr ip6;
+	} ip;
+
 	dns_status status;
 	struct dns_reply ptr_reply;
 	dns_status result = dns_not_found;
-	
-	if (!inet_aton(ipstr, &ip))
-		return dns_failure;
-	
-	status = ptr_lookup(ip, &ptr_reply);
-		
+	dns_status (*lookup)(const char *, struct dns_reply *);
+	int family;
+
+	status = dns_resolve_ipstr(ipstr, &ptr_reply);
 	if (status != dns_success)
 		return status;
 
+	/*
+	 * Since dns_resolve_ipstr succeeded, it is either IPv4 or
+	 * IPv6.
+	 */
+	if (dns_str_is_ipv4(ipstr)) {
+		lookup = a_lookup;
+		family = AF_INET;
+	} else {
+		lookup = aaaa_lookup;
+		family = AF_INET6;
+	}
+
+	if (inet_pton(family, ipstr, &ip) != 1)
+		return dns_failure;
+
 	if (reply)
 		dns_reply_init(reply, dns_reply_str, 0);
 	for (i = 0; i < ptr_reply.count; i++) {
 		struct dns_reply r;
-		status = a_lookup(ptr_reply.data.str[i], &r);
+		status = lookup(ptr_reply.data.str[i], &r);
 		if (status == dns_success) {
 			size_t k;
 
 			for (k = 0; k < r.count; k++) {
-				if (r.data.ip[k] == ip.s_addr) {
+				if (ipaddr_eq(&r, k, &ip) == 0) {
 					result = dns_success;
 					if (reply)
 						dns_reply_push(reply,
@@ -840,15 +1065,24 @@ ptr_validate(const char *ipstr, struct dns_reply *reply)
 		}
 	}
 	dns_reply_free(&ptr_reply);
-	
+
 	return result;
 }
-
 
 mf_status
 dns_to_mf_status(dns_status stat)
 {
-	return (mf_status) stat;
+	static mf_status trans_tab[] = {
+		[dns_success] = mfe_success,
+		[dns_not_found] = mfe_not_found,
+		[dns_failure] = mfe_failure,
+		[dns_temp_failure] = mfe_temp_failure,
+		[dns_too_many] = mfe_too_many
+	};
+
+	if (stat >= NELEMS(trans_tab))
+		return mfe_failure;
+	return trans_tab[stat];
 }
 
 dns_status
@@ -858,21 +1092,22 @@ mf_to_dns_status(mf_status stat)
 }
 
 mf_status
-resolve_ipstr_domain(const char *ipstr, const char *domain, char **phbuf)
+resolve_ipstr(const char *ipstr, char **phbuf)
 {
-	char *hbuf;
 	dns_status dstat;
-		
+	struct dns_reply reply;
+
 	mu_debug(debug_handle, MU_DEBUG_TRACE8,
 		 ("Getting canonical name for %s", ipstr));
 
-	dstat = dns_resolve_ipstr(ipstr, domain, &hbuf);
+	dstat = dns_resolve_ipstr(ipstr, &reply);
 
 	switch (dstat) {
 	case dns_success:
-		mu_debug(debug_handle, MU_DEBUG_TRACE8, 
-			 ("%s resolved to %s", ipstr, hbuf));
-		*phbuf = hbuf;
+		mu_debug(debug_handle, MU_DEBUG_TRACE8,
+			 ("%s resolved to %s", ipstr, reply.data.str[0]));
+		*phbuf = mu_strdup(reply.data.str[0]);
+		dns_reply_free(&reply);
 		break;
 
 	default:
@@ -883,28 +1118,22 @@ resolve_ipstr_domain(const char *ipstr, const char *domain, char **phbuf)
 }
 
 mf_status
-resolve_ipstr(const char *ipstr, char **phbuf)	
-{
-	return resolve_ipstr_domain(ipstr, NULL, phbuf);
-}
-
-mf_status
-resolve_hostname(const char *host, char **pipbuf)	
+resolve_hostname(const char *host, int resolve_family, char **pipbuf)
 {
 	char *ipbuf;
 	dns_status dstat;
 
 	mu_debug(debug_handle, MU_DEBUG_TRACE8,
 		 ("Getting IP address for %s", host));
-		
-	dstat = dns_resolve_hostname(host, &ipbuf);
+
+	dstat = dns_resolve_hostname(host, resolve_family, &ipbuf);
 	switch (dstat) {
 	case dns_success:
-		mu_debug(debug_handle, MU_DEBUG_TRACE8, 
+		mu_debug(debug_handle, MU_DEBUG_TRACE8,
 			 ("%s resolved to %s", host, ipbuf));
 		*pipbuf = ipbuf;
 		break;
-			
+
 	default:
 		mu_debug(debug_handle, MU_DEBUG_TRACE8,
 			 ("%s not resolved", host));
@@ -914,7 +1143,7 @@ resolve_hostname(const char *host, char **pipbuf)
 
 /* Return NS records for the given DOMAIN. */
 dns_status
-ns_lookup(const char *domain, int resolve, struct dns_reply *reply)
+ns_lookup(const char *domain, int resolve_family, struct dns_reply *reply)
 {
 	dns_status status = dns_failure;
 	int rc;
@@ -927,15 +1156,14 @@ ns_lookup(const char *domain, int resolve, struct dns_reply *reply)
 	status = adns_to_dns_status(ans->status);
 	if (status != dns_success)
 		return status;
-	
+
 	dns_reply_init(reply, dns_reply_str, ans->nrrs);
 	for (i = 0; i < ans->nrrs; i++)
 		reply->data.str[i] = mu_strdup(ans->rrs.str[i]);
 	free(ans);
 
-	if (resolve)
-		status = dns_reply_resolve(reply);
-	
+	status = dns_reply_resolve(reply, resolve_family);
+
 	return status;
 }
 
@@ -952,7 +1180,7 @@ struct mu_cfg_param resolver_section_param[] = {
 	{ "config", mu_cfg_callback,
 	  NULL, 0, cb_resolv_conf,
 	  N_("Read DNS configuration from this file (default: /etc/resolv.conf)"),
-          N_("name: string") },
+	  N_("name: string") },
 	{ "max-cname-chain", mu_c_size,
 	  &dns_max_cname_chain, 0, NULL,
 	  N_("Maximum allowed length of CNAME chains") },
@@ -972,4 +1200,3 @@ dnsbase_init(void)
 		mu_cfg_section_add_params(section, resolver_section_param);
 	}
 }
-
