@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2008-2023 Sergey Poznyakoff
+   Copyright (C) 2008-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <mailutils/cli.h>
 #include <mailutils/alloc.h>
 #include <mailutils/cctype.h>
+#include <mailutils/cidr.h>
 #include <sysexits.h>
 #include <pthread.h>
 #include <gacopyz.h>
@@ -103,8 +104,8 @@ struct pmult_priv_data
   char *seid;
   char *seid_c;
   char *taid;
-  unsigned nrcpt;
-  unsigned nbadrcpts;
+  unsigned long nrcpt;
+  unsigned long nbadrcpts;
   enum pmult_msg_state state;
   mu_stream_t hdrstream;
   mu_header_t hdr;
@@ -633,7 +634,7 @@ _cb_define (void *data, mu_config_value_t *arg)
       rc = dc.retcode;
       break;
     }
-  return 0;
+  return rc;
 }
 
 static void
@@ -1413,11 +1414,15 @@ pmult_connect (pmse_ctx_P pmse_ctx, const char *hostname,
   
   if (mu_debug_level_p (pmult_debug, MU_DEBUG_TRACE1))
     {
-      char *p = mu_sockaddr_to_astr (&hostaddr->sa, sizeof *hostaddr);
-      pthread_mutex_lock (&pmult_debug_mutex);
-      mu_debug_log_begin ("Connect from: %s, address %s\n", hostname, p);
-      pthread_mutex_unlock (&pmult_debug_mutex);
-      free (p);
+      char *p;
+      if (mu_sys_sockaddr_format (&p, mu_sockaddr_format_default,
+				  &hostaddr->sa, sizeof *hostaddr) == 0)
+	{
+	  pthread_mutex_lock (&pmult_debug_mutex);
+	  mu_debug_log_begin ("Connect from: %s, address %s\n", hostname, p);
+	  pthread_mutex_unlock (&pmult_debug_mutex);
+	  free (p);
+	}
     }
     
   p = calloc (1, sizeof *p);
