@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2023 Sergey Poznyakoff
+   Copyright (C) 2005-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -441,30 +441,36 @@ smtp_io_capa_find(struct smtp_io_data *io, char const *capa)
 mf_status smtp_io_reopen(struct smtp_io_data *io);
 void smtp_io_close(struct smtp_io_data *io);
 
+enum callout_mode {
+	callout_init,          /* Initial call. */
+	callout_restart,       /* Called after session restart. */
+	callout_restart_ehlo   /* Called to restart from EHLO command. */
+};
+
 static mf_status
-callout_io(struct smtp_io_data *io, const char *hostname, mu_address_t addr)
+callout_io(struct smtp_io_data *io, const char *hostname, mu_address_t addr,
+	   size_t i, enum callout_mode mode)
 {
-	size_t i;
 	size_t mailcount;
-	mf_status status;
+	mf_status status = 0;
+	int rc;
 
-restart:
-	if (io->callback)
-		io->callback(io->callback_closure, "INIT", hostname);
+	if (mode != callout_restart_ehlo) {
+		if (io->callback)
+			io->callback(io->callback_closure, "INIT", hostname);
 
-	mu_address_get_count(addr, &mailcount);
+		if (smtp_recv(io, smtp_timeout_initial))
+			return mf_timeout;
 
-	if (smtp_recv(io, smtp_timeout_initial))
-		return mf_timeout;
+		if (io->callback)
+			io->callback(io->callback_closure, "GRTNG",
+				     smtp_last_received(io));
 
-	if (io->callback)
-		io->callback(io->callback_closure, "GRTNG",
-			     smtp_last_received(io));
+		if (SMTP_MAJOR(io->state.code) != 2)
+			return SMTP_MAJOR(io->state.code) == 4 ?
+				mf_temp_failure : mf_not_found;
+	}
 
-	if (SMTP_MAJOR(io->state.code) != 2)
-		return SMTP_MAJOR(io->state.code) == 4 ?
-			mf_temp_failure : mf_not_found;
-ehlo:
 	smtp_send2(io, "EHLO ", io->ehlo);
 	if (smtp_recv(io, smtp_timeout_helo))
 		return mf_timeout;
@@ -494,10 +500,17 @@ ehlo:
 		}
 
 		if (SMTP_MAJOR(io->state.code) == 2) {
-			int rc = mu_starttls(&io->stream, &tls_conf, MU_TLS_CLIENT);
+			rc = mu_starttls(&io->stream, &tls_conf, MU_TLS_CLIENT);
+			/*
+			 * Clear TLS as a precaution, to make sure we'll
+			 * not get here again, even in the unlikely case if
+			 * the second EHLO still reports STARTTLS.
+			 */
+			io->tls = tls_never;
 			if (rc == 0) {
 				/* Success */
-				goto ehlo;
+				return callout_io(io, hostname, addr,
+						  i, callout_restart_ehlo);
 			} else {
 				/*
 				 * Failure.  According to RFC 3207, abort
@@ -508,15 +521,15 @@ ehlo:
 					mu_error(_("STARTTLS failed (%s)"),
 						 mu_strerror(rc));
 				}
-				io->tls = tls_never;
 				status = smtp_io_reopen(io);
-				if (status != mf_success) {
+				if (status != mf_success || mode != callout_init) {
 					return mf_temp_failure;
 				}
 				if (smtp_transcript) {
 					mu_error("%s", _("restarting session"));
 				}
-				goto restart;
+				return callout_io(io, hostname, addr,
+						  i, callout_restart);
 			}
 		}
 	}
@@ -528,7 +541,17 @@ ehlo:
 	}
 
 	status = mf_success;
-	for (i = 1; i <= mailcount; i++) {
+	if (i == 0) {
+		i = 1;
+		if ((rc = mu_address_get_count(addr, &mailcount)) != 0) {
+			mu_diag_funcall(MU_DIAG_ERROR, "mu_address_get_count",
+					NULL, rc);
+			return mf_temp_failure;
+		}
+	} else {
+		mailcount = i;
+	}
+	for (; i <= mailcount; i++) {
 		const char *fromaddr;
 
 		mu_address_sget_email(addr, i, &fromaddr);
@@ -560,11 +583,6 @@ ehlo:
 			   smtp_io_capa_find(io, "STARTTLS")) {
 			struct smtp_io_state state = smtp_io_state_save(io);
 
-			/* if (reset(io)) { */
-			/* 	smtp_io_state_restore(io, state); */
-			/* 	return mf_timeout; */
-			/* } */
-
 			smtp_send2(io, "STARTTLS", NULL);
 			if (smtp_recv(io, smtp_timeout_mail)) {
 				smtp_io_state_restore(io, state);
@@ -574,7 +592,8 @@ ehlo:
 				int rc = mu_starttls(&io->stream, &tls_conf, MU_TLS_CLIENT);
 				if (rc == 0) {
 					/* Success */
-					goto ehlo;
+					return callout_io(io, hostname, addr,
+							  i, callout_restart_ehlo);
 				} else {
 					if (smtp_transcript) {
 						mu_error(_("STARTTLS failed (%s)"),
@@ -583,7 +602,6 @@ ehlo:
 					smtp_io_close(io);
 					return mf_temp_failure;
 				}
-
 			} else
 				status = mf_temp_failure;
 		} else
@@ -595,7 +613,26 @@ ehlo:
 	smtp_send3(io, "RCPT TO:<", io->email, ">");
 	if (smtp_recv(io, smtp_timeout_rcpt))
 		return mf_timeout;
-	else if (SMTP_MAJOR(io->state.code) != 2)
+	else if (WITH_MAILUTILS_GNUTLS &&
+		 io->tls == tls_ondemand &&
+		 io->state.code == 530 &&
+		 smtp_io_capa_find(io, "STARTTLS")) {
+		/*
+		 * Reportedly, some servers claim that TLS is required after
+		 * RCPT TO command.  Although that contradicts the RFC, try
+		 * to handle that case as well.
+		 * Note, however, that 530 may well mean authentication (not
+		 * TLS) is required.
+		 */
+		struct smtp_io_state state = smtp_io_state_save(io);
+
+		if (reset(io)) {
+			smtp_io_state_restore(io, state);
+			return mf_failure;
+		}
+		io->tls = tls_always;
+		return callout_io(io, hostname, addr, i, callout_restart_ehlo);
+	} else if (SMTP_MAJOR(io->state.code) != 2)
 		return SMTP_MAJOR(io->state.code) == 4 ?
 				mf_temp_failure : mf_not_found;
 	return mf_success;
@@ -770,7 +807,7 @@ callout_host(struct smtp_io_data *io, const char *hostname)
 		return mf_timeout;
 	}
 
-	status = callout_io(io, hostname, addr);
+	status = callout_io(io, hostname, addr, 0, callout_init);
 
 	mu_address_destroy(&addr);
 
