@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2020-2023 Sergey Poznyakoff
+   Copyright (C) 2020-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -68,7 +68,7 @@ report_pem_error(char const *priv_file, int rc, int ec)
 		break;
 
 	case READ_PEM_EOF:
-		mu_error(_("unexpected end of file in file %s"), priv_file);
+		mu_error(_("unexpected end of file in %s"), priv_file);
 		break;
 
 	case READ_PEM_BADVER:
