@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2023 Sergey Poznyakoff
+   Copyright (C) 2005-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ static int ctx_setreply(void *data, char *code, char *xcode, char *message);
 
 struct message_data {
 	eval_environ_t env;        /* Evaluation environment */
-	char *helostr;             /* Domain name obtained in HELO phase */   
+	char *helostr;             /* Domain name obtained in HELO phase */
 	char msgid[64];            /* Message ID */
 };
 static struct message_data *test_message_data;
@@ -71,15 +71,15 @@ priv_get(SMFICTX *ctx)
 		md = test_message_data;
 	else
 		md = (struct message_data*) gacopyz_getpriv(ctx);
-		
+
 	if (!md) {
 		md = malloc(sizeof(*md));
-		if (!md) 
+		if (!md)
 			mu_error(_("not enough memory"));
 		else {
 			milter_sockaddr_t addr;
 			socklen_t len = sizeof(addr);
-		
+
 			md->env = create_environment(ctx,
 						     ctx_getsym,
 						     ctx_setreply,
@@ -141,7 +141,7 @@ ctx_getsym(void *data, const char *str)
 static int
 ctx_setreply(void *data, char *code, char *xcode, char *message)
 {
-	if (code) 
+	if (code)
 		return gacopyz_setreply(data, code, xcode, message);
 	return 0;
 }
@@ -171,7 +171,7 @@ capture_from(eval_environ_t env, const char *str)
 		char datebuf[26];
 
 		env_capture_start(env);
-		
+
 		t = time(NULL);
 		tm = localtime(&t);
 		mu_strftime(datebuf, sizeof datebuf,
@@ -204,7 +204,7 @@ static void
 capture_eom(eval_environ_t env)
 {
 }
-	
+
 
 /* Cleanup functions */
 void
@@ -233,7 +233,7 @@ xeval(eval_environ_t env, enum smtp_state tag)
 	env_make_frame(env);
 	set_milter_state(env, tag);
 	rc = eval_environment(env, entry_point[tag]);
-	if (rc) 
+	if (rc)
 		mu_error(_("execution of the filter program was not finished"));
 	return rc;
 }
@@ -273,52 +273,57 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *hostaddr)
 	int port;
 	char *addrstr;
 	int family;
-	
+
 	if (!hostaddr) {
 		family = MFAM_STDIO;
 		port = 0;
 		addrstr = "";
-	} else switch (hostaddr->sa.sa_family) {
-	case PF_INET:
-		family = MFAM_INET;
-		port = ntohs(hostaddr->sin.sin_port);
-		addrstr = inet_ntoa(hostaddr->sin.sin_addr);
-		break;
+		env_save_socket_family(md->env, AF_INET);
+	} else {
+		switch (hostaddr->sa.sa_family) {
+		case PF_INET:
+			family = MFAM_INET;
+			port = ntohs(hostaddr->sin.sin_port);
+			addrstr = inet_ntoa(hostaddr->sin.sin_addr);
+			break;
 
-	case PF_INET6: {
-		char host[NI_MAXHOST];
+		case PF_INET6: {
+			char host[NI_MAXHOST];
 
-		family = MFAM_INET6;
-		port = ntohs(hostaddr->sin6.sin6_port);
-		if (getnameinfo(&hostaddr->sa, sizeof(hostaddr->sin6),
-				host, sizeof host,
-				NULL, 0,
-				NI_NUMERICHOST) == 0)
-			addrstr = host;
-		else
-			addrstr = "[unresolved]";
-		break;
+			family = MFAM_INET6;
+			port = ntohs(hostaddr->sin6.sin6_port);
+			if (getnameinfo(&hostaddr->sa, sizeof(hostaddr->sin6),
+					host, sizeof host,
+					NULL, 0,
+					NI_NUMERICHOST) == 0)
+				addrstr = host;
+			else
+				addrstr = "[unresolved]";
+			break;
+		}
+
+		case PF_UNIX:
+			family = MFAM_UNIX;
+			port = 0;
+			addrstr = hostaddr->sunix.sun_path;
+			break;
+
+		default:
+			mu_error(_("mlfi_connect: unsupported address family: %d"),
+				 hostaddr->sa.sa_family);
+			gacopyz_setreply(ctx, "410", NULL,
+					 "Local configuration error; please try again later");
+			return SMFIS_TEMPFAIL;
+		}
+		env_save_socket_family(md->env, hostaddr->sa.sa_family);
 	}
-		
-	case PF_UNIX:
-		family = MFAM_UNIX;
-		port = 0;
-		addrstr = hostaddr->sunix.sun_path;
-		break;
 
-	default:
-		mu_error(_("mlfi_connect: unsupported address family: %d"),
-			 hostaddr->sa.sa_family);
-		gacopyz_setreply(ctx, "410", NULL,
-				 "Local configuration error; please try again later");
-		return SMFIS_TEMPFAIL;
-	}
-	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, 
-              ("Processing xxfi_connect: %s, %d, %s, %u",
-	        hostname, family, addrstr, port));
+	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7,
+		 ("Processing xxfi_connect: %s, %d, %s, %u",
+		  hostname, family, addrstr, port));
 	mf_proctitle_format("Connect: %s, %d, %s, %u",
 			    hostname, family, addrstr, port);
-	
+
 	env_init(md->env);
 	env_push_string(md->env, addrstr);
 	env_push_number(md->env, port);
@@ -368,10 +373,10 @@ mlfi_envfrom(SMFICTX *ctx, char **argv)
 	struct message_data *md = priv_get(ctx);
 	char *p = concat_args(argv + 1);
 
-	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, 
-              ("Processing xxfi_envfrom: %s %s", argv[0], p));
+	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7,
+	      ("Processing xxfi_envfrom: %s %s", argv[0], p));
 	mf_proctitle_format("%sMAIL FROM %s %s", md->msgid, argv[0], p);
-	
+
 	env_init(md->env);
 	capture_from(md->env, argv[0]);
 	env_push_string(md->env, p);
@@ -386,10 +391,10 @@ sfsistat
 mlfi_envrcpt(SMFICTX *ctx, char ** argv)
 {
 	sfsistat status;
-        struct message_data *md = priv_get(ctx);
+	struct message_data *md = priv_get(ctx);
 	char *p = concat_args(argv + 1);
 
-	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, 
+	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7,
 		 ("Processing xxfi_envrcpt: %s %s", argv[0], p));
 	mf_proctitle_format("%sRCPT TO %s %s", md->msgid, argv[0], p);
 	env_init(md->env);
@@ -438,7 +443,7 @@ sfsistat
 mlfi_eoh(SMFICTX *ctx)
 {
 	sfsistat status;
-        struct message_data *md = priv_get(ctx);
+	struct message_data *md = priv_get(ctx);
 	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, ("Processing xxfi_eoh"));
 	mf_proctitle_format("%sEOH", md->msgid);
 	env_init(md->env);
@@ -453,7 +458,7 @@ mlfi_body(SMFICTX *ctx, unsigned char *bodyp, size_t len)
 {
 	sfsistat status;
 	struct message_data *md = priv_get(ctx);
-	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, 
+	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7,
 		 ("Processing xxfi_body: %lu", (unsigned long) len));
 	mf_proctitle_format("%sBODY", md->msgid);
 	env_init(md->env);
@@ -493,9 +498,9 @@ xlate_and_replace_body(SMFICTX *ctx, const char *value, size_t size)
 	}
 
 	while (size) {
-	        size_t n = mem_search(value, '\n', size);
+		size_t n = mem_search(value, '\n', size);
 		size_t off;
-		
+
 		if (value[n] == '\n') {
 			off = n + 1;
 			if (n > 0 && value[n-1] == '\r')
@@ -509,7 +514,7 @@ xlate_and_replace_body(SMFICTX *ctx, const char *value, size_t size)
 				 mu_strerror(rc));
 			break;
 		}
-		
+
 		value += off;
 		size -= off;
 	}
@@ -552,7 +557,7 @@ run_msgmod(void *item, void *data)
 {
 	struct msgmod_closure *hdr = item;
 	SMFICTX *ctx = data;
-	
+
 	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE6,
 		 ("%s %s: %s %u",
 		  msgmod_opcode_str(hdr->opcode),
@@ -562,7 +567,7 @@ run_msgmod(void *item, void *data)
 	case header_add:
 		gacopyz_add_header(ctx, hdr->name, hdr->value);
 		break;
-		
+
 	case header_replace:
 		gacopyz_change_header(ctx, hdr->idx, hdr->name, hdr->value);
 		break;
@@ -574,7 +579,7 @@ run_msgmod(void *item, void *data)
 	case header_insert:
 		gacopyz_insert_header(ctx, hdr->idx, hdr->name, hdr->value);
 		break;
-		
+
 	case rcpt_add:
 		gacopyz_add_rcpt(ctx, hdr->name);
 		break;
@@ -582,7 +587,7 @@ run_msgmod(void *item, void *data)
 	case rcpt_delete:
 		gacopyz_del_rcpt(ctx, hdr->name);
 		break;
-		
+
 	case quarantine:
 		gacopyz_quarantine(ctx, hdr->name);
 		break;
@@ -599,11 +604,11 @@ run_msgmod(void *item, void *data)
 			break;
 		}
 		break;
-		
+
 	case set_from:
 		gacopyz_chgfrom(ctx, hdr->name, hdr->value);
 		break;
-		
+
 	default:
 		abort();
 	}
@@ -613,7 +618,7 @@ run_msgmod(void *item, void *data)
 sfsistat
 mlfi_eom(SMFICTX *ctx)
 {
-        sfsistat status;
+	sfsistat status;
 	struct message_data *md = priv_get(ctx);
 
 	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, ("Processing xxfi_eom"));
@@ -629,7 +634,7 @@ mlfi_eom(SMFICTX *ctx)
 			 ("flushing message modification queue"));
 		env_msgmod_apply(md->env, run_msgmod, ctx);
 	}
-	
+
 	mf_proctitle_format("%sfinished", md->msgid);
 	clear_rcpt_count(md->env);
 	env_msgmod_clear(md->env);
@@ -644,14 +649,14 @@ mlfi_abort(SMFICTX *ctx)
 
 	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, ("Abort"));
 	mf_proctitle_format("%saborting", md->msgid);
-        env_msgmod_clear(md->env);
-        md->msgid[0] = 0;
+	env_msgmod_clear(md->env);
+	md->msgid[0] = 0;
 	/* Note: the value of helostr is not discarded: RFC 2822,
 	   section 4.1.1.5 states that RSET issued immediately
 	   after EHLO is effectively equivalent to a NOOP. */
 	env_init_dataseg(md->env);
 	return SMFIS_CONTINUE;
-}		
+}
 
 sfsistat
 mlfi_close(SMFICTX *ctx)
@@ -659,7 +664,7 @@ mlfi_close(SMFICTX *ctx)
 	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, ("Close"));
 	mf_proctitle_format("closing");
 	filter_cleanup(ctx);
-	return SMFIS_CONTINUE; 
+	return SMFIS_CONTINUE;
 }
 
 static int
@@ -692,19 +697,19 @@ mlfi_negotiate(SMFICTX *ctx,
 {
 	enum gacopyz_stage i;
 
-	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7, 
+	mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE7,
 		 ("Negotiate: mta_actions=%#lx, mta_capa=%#lx, "
 		  "filter_actions=%#lx, filter_capa=%#lx",
 		  mta_actions, mta_capa, *filter_actions, *filter_capa));
 
 	if (first_used_state >= smtp_state_first
 	    && first_used_state <= smtp_state_last) {
-		mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE6, 
+		mu_debug(MF_SOURCE_ENGINE, MU_DEBUG_TRACE6,
 			 ("Registering \"i\" macro for state %s",
 			  xstate_to_string(first_used_state)));
-		register_macro(first_used_state, "i"); 
+		register_macro(first_used_state, "i");
 	}
-	
+
 	for (i = 0; i < gacopyz_stage_max; i++) {
 		char *str = get_stage_macro_string(i);
 		gacopyz_setsymlist(ctx, i, str);
@@ -715,16 +720,16 @@ mlfi_negotiate(SMFICTX *ctx,
 
 static struct smfiDesc smfilter =
 {
-	"MailfromFilter",	
-	SMFI_VERSION,	
+	"MailfromFilter",
+	SMFI_VERSION,
 	SMFI_V2_ACTS|SMFIF_CHGFROM, /* FIXME: Add flags as needed */
-	NULL,	        /* connection info filter */
-	NULL,	        /* SMTP HELO command filter */
-	NULL,	        /* envelope sender filter */
-	NULL,	        /* envelope recipient filter */
-	NULL,	        /* header filter */
+	mlfi_connect,	/* connection info filter */
+	NULL,		/* SMTP HELO command filter */
+	NULL,		/* envelope sender filter */
+	NULL,		/* envelope recipient filter */
+	NULL,		/* header filter */
 	NULL,           /* end of header */
-	NULL,	        /* body block filter */
+	NULL,		/* body block filter */
 	mlfi_eom,	/* end of message */
 	mlfi_abort,	/* message aborted */
 	mlfi_close,	/* connection cleanup */
@@ -732,7 +737,7 @@ static struct smfiDesc smfilter =
 	NULL,   /* unknown command handler */
 	NULL,   /* data handler */
 	mlfi_negotiate,   /* negotiate */
-	
+
 	child_start,   /* child start */
 	NULL,   /* child finish */
 	NULL,   /* idle callback */
@@ -798,12 +803,12 @@ milter_enable_state(enum smtp_state state)
 	if (state >= smtp_state_first && state <= smtp_state_last
 	    && (first_used_state == smtp_state_none || state < first_used_state))
 		first_used_state = state;
-	
+
 	switch (state) {
 	case smtp_state_connect:
 		smfilter.xxfi_connect = mlfi_connect;
 		break;
-		
+
 	case smtp_state_helo:
 		smfilter.xxfi_helo = mlfi_helo;
 		break;
@@ -811,7 +816,7 @@ milter_enable_state(enum smtp_state state)
 	case smtp_state_envfrom:
 		smfilter.xxfi_envfrom = mlfi_envfrom;
 		break;
-		
+
 	case smtp_state_envrcpt:
 		smfilter.xxfi_envrcpt = mlfi_envrcpt;
 		break;
@@ -819,7 +824,7 @@ milter_enable_state(enum smtp_state state)
 	case smtp_state_data:
 		smfilter.xxfi_data = mlfi_data;
 		break;
-		
+
 	case smtp_state_header:
 		smfilter.xxfi_header = mlfi_header;
 		break;
@@ -827,7 +832,7 @@ milter_enable_state(enum smtp_state state)
 	case smtp_state_eoh:
 		smfilter.xxfi_eoh = mlfi_eoh;
 		break;
-		
+
 	case smtp_state_body:
 		smfilter.xxfi_body = mlfi_body;
 		break;
@@ -836,4 +841,3 @@ milter_enable_state(enum smtp_state state)
 		break;
 	}
 }
-
