@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2023 Sergey Poznyakoff
+   Copyright (C) 2005-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@ mf_status dns_to_mf_status(dns_status s);
 mf_status resolve_ipstr(const char *ipstr, char **hbuf);
 mf_status resolve_ipstr_domain(const char *ipstr, const char *domain,
 			       char **phbuf);
-mf_status resolve_hostname(const char *host, char **pipbuf);
+mf_status resolve_hostname(const char *host, int resolve_hostname, char **pipbuf);
 	
 
 /* Debugging macros */
@@ -912,7 +912,8 @@ void pp_define(const char *symbol);
 void test_message_data_init(eval_environ_t env);
 
 void mailfromd_test(int argc, char **argv);
-void mailfromd_run(prog_counter_t entry, int argc, char **argv);
+void mailfromd_run(prog_counter_t entry, int argc, char **argv,
+		   int macc, char **macv);
 
 void milter_enable_state(enum smtp_state state);
 int milter_session_server(const char *id,
