@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2005-2023 Sergey Poznyakoff
+   Copyright (C) 2005-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@ pathbuf_add(struct pathbuf *pb, char const *dir)
 	}
 }
 
-char const *default_suffixes[] = { ".mfl", ".mf", NULL };
+char const *default_suffixes[] = { ".mfl", NULL };
 
 struct pathbuf include_path = PATHBUF_INITIALIZER(
 	DEFAULT_MODDIR "/include:"
@@ -288,15 +288,8 @@ require_module(const char *modname, struct import_rule *import_rules)
 {
 	char *pathname;
 	
-	if (try_file(modname, default_suffixes, &module_path, 0, 0, &pathname) == 0) {
-		if (try_file(modname, default_suffixes, &include_path, 0, 1, &pathname) == 0) {
-			return;
-		} else {
-			parse_warning(_("module %s found in deprecated location: include path (%s)"), modname, pathname);
-			parse_warning(_("please move it somewhere to your module path (%s)"),
-				      module_path.base);
-		}
-	}
+	if (try_file(modname, default_suffixes, &module_path, 0, 1, &pathname) == 0)
+		return;
 	begin_module(modname, pathname, import_rules);
 }
 
