@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2006-2023 Sergey Poznyakoff
+   Copyright (C) 2006-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -297,6 +297,7 @@ struct eval_environ {
 	
 	/* Sendmail interaction data: */
 	SMFICTX *ctx;          /* Milter Context */
+	int conn_family;       /* SMTP connection family. */
 	void *data;            /* MTA symbol table */
 	/* methods to access the latter */
 	const char *(*getsym)(void *data, const char *str);
@@ -3038,6 +3039,18 @@ env_builtin_priv_destroy(eval_environ_t env)
 	}
 }
 
+void
+env_save_socket_family(eval_environ_t env, int family)
+{
+	env->conn_family = family;
+}
+
+int
+env_socket_family(eval_environ_t env)
+{
+	return env->conn_family;
+}
+
 eval_environ_t
 create_environment(SMFICTX *ctx,
 		   const char *(*getsym)(void *data, const char *str),
@@ -3063,6 +3076,11 @@ create_environment(SMFICTX *ctx,
 	init_dataseg(env->dataseg, datasize);
 	
 	env->ctx = ctx;
+	/*
+	 * Assume inet address family.  It will be replaced with the
+	 * actual value when mlfi_connect is called.
+	 */
+	env->conn_family = AF_INET;
 	env->data = data;
 	env->getsym = getsym;
 	env->setreply = setreply;
