@@ -1,5 +1,5 @@
 /* This file is part of Mailfromd.
-   Copyright (C) 2007-2023 Sergey Poznyakoff
+   Copyright (C) 2007-2024 Sergey Poznyakoff
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,13 +30,13 @@ typedef struct spf_term spf_term;
 typedef struct spf_data {
 	/* Internal data */
 	mu_opool_t tmpool;       /* Opool for keeping temporary values */
-	char *buffer;            /* Expansion and transformation buffer */  
+	char *buffer;            /* Expansion and transformation buffer */
 	size_t bufsize;          /* Size of buffer */
 	spf_term *mod[MAX_SPF_MOD];
 	size_t loopno;
 	/* Input data */
 	spf_query_t q;
-	struct in_addr ipaddr;   /* Originator IP in binary (network order) */
+	struct mu_cidr addr;     /* Originator IP as CIDR */
 	/* Output data */
 	spf_result result;       /* SPF result if a term throws an exception */
 	spf_answer_t *answer;
@@ -59,15 +59,30 @@ int
 spf_data_init(struct spf_data *dat, spf_query_t *q, spf_answer_t *answer,
 	      size_t loopno)
 {
+	struct addrinfo *res, hints;
+	int rc;
+
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_flags = AI_NUMERICHOST;
+	rc = getaddrinfo(q->ipstr, NULL, &hints, &res);
+	if (rc) {
+		mu_error(_("spf_data_init: %s: %s"),
+			 q->ipstr, gai_strerror(rc));
+		return 1;
+	}
+
 	memset(dat, 0, sizeof dat[0]);
 	mu_opool_create(&dat->tmpool, MU_OPOOL_ENOMEMABRT);
 	dat->q = *q;
-	if (!inet_aton(dat->q.ipstr, &dat->ipaddr)) {
-		mu_error(_("spf_data_init: invalid IP address: %s"),
-			 dat->q.ipstr);
+
+	rc = mu_cidr_from_sockaddr(&dat->addr, res->ai_addr);
+	freeaddrinfo(res);
+	if (rc) {
+		mu_error(_("spf_data_init: can't convert IP address %s: %s"),
+			 q->ipstr, mu_strerror(rc));
 		return 1;
 	}
-	dat->ipaddr.s_addr = ntohl(dat->ipaddr.s_addr);
+
 	dat->answer = answer;
 	dat->loopno = loopno;
 	return 0;
@@ -151,7 +166,7 @@ spf_reverse(char *buf, size_t len, char const *delim)
 	size_t tmpsize = 0;
 	size_t delta;
 	size_t bufsize = strlen(buf);
-	
+
 	do {
 		lp = scanforw(buf, delim);
 		if (!lp)
@@ -159,7 +174,7 @@ spf_reverse(char *buf, size_t len, char const *delim)
 		llen = lp - buf;
 		rp = scanback(buf, delim, bufsize) + 1;
 		rlen = bufsize - (rp - buf);
-		
+
 		if (llen == rlen)
 			swapi(buf, rp, llen);
 		else if (llen < rlen) {
@@ -187,7 +202,7 @@ spf_reverse(char *buf, size_t len, char const *delim)
 			memcpy(rp, tmp, delta);
 			lp -= delta;
 		}
- 		buf = lp + 1;
+		buf = lp + 1;
 		bufsize = rp - buf - 1;
 	} while (rp > lp + 1);
 	free(tmp);
@@ -246,10 +261,10 @@ spf_transform(struct spf_data *dat, const char *x, size_t l, const char **pptr)
 	int reverse = 0;
 	char delim[sizeof(DELIMITERS)] = ".";
 	int dlen;
-	
+
 	while (mu_isdigit(*p))
 		nparts = nparts * 10 + *p++ - '0';
-	
+
 	if (*p == 'r') {
 		p++;
 		reverse = 1;
@@ -265,7 +280,7 @@ spf_transform(struct spf_data *dat, const char *x, size_t l, const char **pptr)
 	if (*p != '}')
 		return 1;
 	*pptr = p + 1;
-		
+
 	if (l == 0)
 		l = strlen(x);
 
@@ -305,6 +320,26 @@ domain_match(const char *name, const char *domain)
 	}
 }
 
+static int
+to_xdig(unsigned char b)
+{
+	if (b >= 0xa)
+		return 'a' + b - 0xa;
+	else
+		return '0' + b;
+}
+
+/*
+ * Max. buffer capacity for dotted-quad IPv4 is 4*4 = 16 bytes (four
+ * three-digit segments, each terminated by a dot, or \0 for the last
+ * one).
+ * Max. buffer capacity for IPv6 addresses is 16*3 = 48 bytes (sixteen
+ * 2-hexdigit segments, each terminated by a dot, or \0 for the last
+ * one).
+ * IP buffer must be able to accomodate the larger of the two:
+ */
+#define IPBUF_DOTTED_SIZE (16*3)
+
 /* Expand a single macro as per RFC 4408, chapter 8.
    PPTR points to the macro symbol, right past the opening curly brace.
    ALLOW_EXP is 1 if the expansion of macros c, r and t is allowed (see
@@ -316,20 +351,35 @@ int
 spf_expand_do(struct spf_data *dat, int allow_exp, const char **pptr)
 {
 	char *q;
-	
+
 	switch (*(*pptr)++) {
 	case 'd':
 	case 'D':
 		return spf_transform(dat, dat->q.domain, 0, pptr);
-		
+
 	case 'h':
 	case 'H':
 		return spf_transform(dat, dat->q.helo_domain, 0, pptr);
-		
+
 	case 'i':
 	case 'I':
-		return spf_transform(dat, dat->q.ipstr, 0, pptr);
-		
+		if (dat->addr.family == AF_INET6) {
+			char ipbuf[IPBUF_DOTTED_SIZE], *p;
+			int i;
+
+			/* For IPv6 addresses, the "i" macro expands to a
+			   dot-format address. */
+			p = ipbuf;
+			for (i = 0; i < dat->addr.len; i++) {
+				if (i > 0)
+					*p++ = '.';
+				*p++ = to_xdig(dat->addr.address[i] >> 4);
+				*p++ = to_xdig(dat->addr.address[i] & 0xf);
+			}
+			return spf_transform(dat, ipbuf, 0, pptr);
+		} else
+			return spf_transform(dat, dat->q.ipstr, 0, pptr);
+
 	case 'l':
 	case 'L':
 		q = strchr(dat->q.sender, '@');
@@ -337,14 +387,14 @@ spf_expand_do(struct spf_data *dat, int allow_exp, const char **pptr)
 			return 1;
 		return spf_transform(dat, dat->q.sender, q - dat->q.sender,
 				     pptr);
-		
+
 	case 'o':
 	case 'O':
 		q = strchr(dat->q.sender, '@');
 		if (!q)
 			return 1;
 		return spf_transform(dat, q + 1, 0, pptr);
-		
+
 	case 'p':
 	case 'P':
 	{
@@ -366,7 +416,7 @@ spf_expand_do(struct spf_data *dat, int allow_exp, const char **pptr)
 			for (i = 0; i < reply.count; i++)
 				if (domain_match(reply.data.str[i],
 						 dat->q.domain)) {
-					name = reply.data.str[i]; 
+					name = reply.data.str[i];
 					break;
 				}
 
@@ -378,7 +428,7 @@ spf_expand_do(struct spf_data *dat, int allow_exp, const char **pptr)
 		dns_reply_free(&reply);
 		return rc;
 	}
-		
+
 	case 's':
 	case 'S':
 		return spf_transform(dat, dat->q.sender, 0, pptr);
@@ -387,21 +437,31 @@ spf_expand_do(struct spf_data *dat, int allow_exp, const char **pptr)
 	case 'V':
 		/* FIXME: add IPv6 support */
 		return spf_transform(dat, "in-addr", 0, pptr);
-		
+
 	case 'c':
 	case 'C':
 		if (!allow_exp)
 			return 1;
-		else
-			return spf_transform(dat, dat->q.ipstr, 0, pptr);
-		
+		else {
+			/*
+			 * [For IPv6 addresses, ] [t]he "c" macro can expand to
+			 * any of the hexadecimal colon-format addresses
+			 * specified in Section 2.2 of [RFC4291].  It is
+			 * intended for humans to read.
+			 */
+			char buf[MU_CIDR_MAXBUFSIZE];
+			mu_cidr_to_string(&dat->addr, MU_CIDR_FMT_ADDRONLY,
+					  buf, sizeof(buf), NULL);
+			return spf_transform(dat, buf, 0, pptr);
+		}
+
 	case 'r':
 	case 'R':
 		if (!allow_exp)
 			return 1;
 		else
 			return spf_transform(dat, dat->q.my_domain, 0, pptr);
-		
+
 	case 't':
 	case 'T':
 		if (!allow_exp)
@@ -429,7 +489,7 @@ int
 _spf_macro_expand(const char *input, struct spf_data *dat, int allow_exp)
 {
 	const char *p;
-	
+
 	while (p = strchr(input, '%')) {
 		size_t len = p - input;
 		if (len > 0)
@@ -440,17 +500,17 @@ _spf_macro_expand(const char *input, struct spf_data *dat, int allow_exp)
 			if (spf_expand_do(dat, allow_exp, &p))
 				return 1;
 			break;
-			
+
 		case '%':
 			mu_opool_append_char(dat->tmpool, '%');
 			p += 2;
 			break;
-			
+
 		case '_':
 			mu_opool_append_char(dat->tmpool, ' ');
 			p += 2;
 			break;
-			
+
 		case '-':
 			mu_opool_appendz(dat->tmpool, "%20");
 			p += 2;
@@ -461,7 +521,7 @@ _spf_macro_expand(const char *input, struct spf_data *dat, int allow_exp)
 		}
 		input = p;
 	}
-	if (input) 
+	if (input)
 		mu_opool_appendz(dat->tmpool, input);
 	mu_opool_append_char(dat->tmpool, 0);
 	return 0;
@@ -498,8 +558,7 @@ typedef struct {
 	spf_arg_type type;
 	union {
 		char *domain_spec;
-		struct in_addr ip;
-		/* FIXME: ipv6 */
+		struct mu_cidr ip;
 	} v;
 } spf_term_arg;
 
@@ -515,7 +574,7 @@ typedef spf_term_result (*spf_term_handler)(struct spf_data *dat,
 
 struct spf_term {
 	spf_term_type type;           /* Term type */
-	const char *expr;             /* Original expression (for debugging)*/ 
+	const char *expr;             /* Original expression (for debugging)*/
 	spf_term_handler handler;     /* Term handler */
 	int has_arg;                  /* Is an explicit argument given */
 	spf_term_arg arg;             /* Argument if has_arg==1 */
@@ -543,7 +602,7 @@ struct spf_term_syntax {
 	case dns_temp_failure:                                    \
 		dat->result = spf_temp_error;                     \
 		return spf_term_exception;                        \
-	}		
+	}
 
 
 /* Handlers for particular terms */
@@ -551,7 +610,7 @@ struct spf_term_syntax {
 /* RFC 4408, 5.1.
      all              = "all"
 */
-static spf_term_result 
+static spf_term_result
 mech_all(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
 	dat->mod[SPF_MOD_REDIRECT] = NULL;
@@ -559,16 +618,16 @@ mech_all(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 }
 
 /* 5.2.
-         include          = "include"  ":" domain-spec
+	 include          = "include"  ":" domain-spec
 */
-static spf_term_result 
+static spf_term_result
 mech_include(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
 	spf_result res;
 	spf_term_result tres;
 	spf_query_t query;
 	size_t mechn;
-	
+
 	if (!arg) {
 		mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
 			 ("include used without argument"));
@@ -582,13 +641,13 @@ mech_include(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 	query.domain = arg->v.domain_spec;
 	mechn = dat->answer->mechn;
 	res = spf_check_host_internal(&query, dat->answer, dat->loopno);
-	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1, 
+	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1,
 		 ("check_host returned %s", spf_result_str[res]));
 	switch (res) {
 	case spf_pass:
 		tres = spf_term_match;
 		break;
-		
+
 	case spf_fail:
 	case spf_soft_fail:
 	case spf_neutral:
@@ -600,65 +659,89 @@ mech_include(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 		break;
 
 	case spf_temp_error:
-		dat->result = spf_temp_error;                     
+		dat->result = spf_temp_error;
 		tres = spf_term_exception;
 		break;
-		
+
 	case spf_perm_error:
 	case spf_none:
 		dat->result = spf_perm_error;
 		tres = spf_term_exception;
 		break;
-	}		
+	}
 
 	return tres;
 }
 
-/* Compute IPv4 netmask for the given length */
-static unsigned long
-make_netmask(unsigned long masklen)
+static char *
+cidr_as_string(struct mu_cidr *cidr, char *buf, size_t bufsize)
 {
-	unsigned long netmask;
-	masklen = 32 - masklen;
-	if (masklen == 32)
-		netmask = 0;
-	else
-		netmask = (0xfffffffful >> masklen) << masklen;
-	return netmask;
+	if (mu_cidr_to_string(cidr, MU_CIDR_FMT_SIMPLIFY, buf, bufsize, NULL)) {
+		if (bufsize >= 2) {
+			buf[0] = '?';
+			buf[1] = 0;
+		} else
+			buf[0] = 0;
+	}
+	return buf;
+}
+
+static void
+fill_cidr_netmask(struct mu_cidr *cidr, unsigned long masklen)
+{
+	int i, cnt;
+
+	cnt = masklen / 8;
+	for (i = 0; i < cnt; i++)
+		cidr->netmask[i] = 0xff;
+	if (i < MU_INADDR_BYTES) {
+		cnt = 8 - masklen % 8;
+		cidr->netmask[i++] = (0xff >> cnt) << cnt;
+		for (; i < MU_INADDR_BYTES; i++)
+			cidr->netmask[i] = 0;
+	}
+	for (i = 0; i < MU_INADDR_BYTES; i++)
+		cidr->address[i] &= cidr->netmask[i];
+}
+
+static int
+ip_matches(int family, void *ip, unsigned long masklen, struct mu_cidr const *addr)
+{
+	struct mu_cidr t;
+
+	t.family = family;
+	t.len = _mu_inaddr_to_bytes(t.family, ip, t.address);
+	fill_cidr_netmask(&t, masklen);
+	return mu_cidr_match(&t, (struct mu_cidr*)addr) == 0;
 }
 
 /* 5.3.
       A                = "a"      [ ":" domain-spec ] [ dual-cidr-length ]
 */
-static spf_term_result 
+static spf_term_result
 mech_a(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
 	const char *domain_spec;
-	unsigned long netmask;
-	struct in_addr addr;
 	struct dns_reply r;
 	size_t i;
 	spf_term_result res;
-	
-	if (arg) 
+
+	if (arg)
 		domain_spec = arg->v.domain_spec;
-	else 
+	else
 		domain_spec = dat->q.domain;
 
-	netmask = make_netmask(masklen);
-	
-	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1, 
-		 ("A domain_spec=%s, netmask=%lx", domain_spec, netmask));
-	
-	DNS_CATCH(a_lookup(domain_spec, &r));
-	
-	addr.s_addr = dat->ipaddr.s_addr & netmask;
-	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE6,
-		 ("A: s_addr=%x", addr.s_addr));
+	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1,
+		 ("A domain_spec=%s, masklen=%lu", domain_spec, masklen));
+
+	DNS_CATCH((dat->addr.family == AF_INET6 ? aaaa_lookup : a_lookup)(domain_spec, &r));
 
 	res = spf_term_nomatch;
 	for (i = 0; i < r.count; i++) {
-		if ((ntohl(r.data.ip[i]) & netmask) == addr.s_addr) {
+		if (ip_matches(dat->addr.family,
+			       dat->addr.family == AF_INET6
+			        ? (void*)&r.data.ip6[i]
+			        : (void*)&r.data.ip[i], masklen, &dat->addr)) {
 			res = spf_term_match;
 			break;
 		}
@@ -672,30 +755,29 @@ mech_a(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 /* 5.4.
       MX               = "mx"     [ ":" domain-spec ] [ dual-cidr-length ]
 */
-static spf_term_result  
+static spf_term_result
 mech_mx(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
-	unsigned long netmask = make_netmask(masklen);
 	struct dns_reply reply;
 	size_t i;
 	spf_term_result result = spf_term_nomatch;
 	const char *domain_spec;
-	
+
 	if (arg)
 		domain_spec = arg->v.domain_spec;
 	else
 		domain_spec = dat->q.domain;
 
 	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1,
-		 ("MX domain_spec=%s, netmask=%lx",
-		  domain_spec, netmask));
+		 ("MX domain_spec=%s, masklen=%lu",
+		  domain_spec, masklen));
+
+	DNS_CATCH(mx_lookup(domain_spec, resolve_none, &reply));
 
-	DNS_CATCH(mx_lookup(domain_spec, 0, &reply));
-	
 	for (i = 0; i < reply.count; i++) {
 		spf_term_arg targ;
 		spf_term_result res;
-		
+
 		targ.type = spf_arg_domain_spec;
 		targ.v.domain_spec = reply.data.str[i];
 		res = mech_a(dat, &targ, masklen);
@@ -712,21 +794,21 @@ mech_mx(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 /* 5.5.
       PTR              = "ptr"    [ ":" domain-spec ]
 */
-static spf_term_result  
+static spf_term_result
 mech_ptr(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
 	struct dns_reply reply;
 	size_t i;
 	const char *domain_spec;
 	spf_term_result result = spf_term_nomatch;
-	
+
 	if (arg)
 		domain_spec = arg->v.domain_spec;
 	else
 		domain_spec = dat->q.domain;
-	
+
 	DNS_CATCH(ptr_validate(dat->q.ipstr, &reply));
-	
+
 	for (i = 0; i < reply.count; i++)
 		if (domain_match(reply.data.str[i], domain_spec)) {
 			result = spf_term_match;
@@ -738,43 +820,36 @@ mech_ptr(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 
 /* 5.6.
       IP4              = "ip4"      ":" ip4-network   [ ip4-cidr-length ]
+      IP6              = "ip6"      ":" ip6-network   [ ip6-cidr-length ]
 */
-static spf_term_result  
-mech_ip4(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
+static spf_term_result
+mech_ip(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
-	unsigned long netmask = make_netmask(masklen);
+	char ipstr[MU_CIDR_MAXBUFSIZE];
+	struct mu_cidr cidr;
 
 	if (!arg) {
 		mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
-			 ("ip4 used without argument"));
+			 ("ip4/ip6 used without argument"));
 		dat->result = spf_perm_error;
 		return spf_term_exception;
 	}
-	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1, 
-		 ("IP4 addr=%lx, netmask=%lx",
-		  (unsigned long) arg->v.ip.s_addr, netmask));
+	cidr = arg->v.ip;
+	fill_cidr_netmask(&cidr, masklen);
+	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1,
+		 ("IP%d: %s",
+		  cidr.family == AF_INET6 ? 6 : 4,
+		  cidr_as_string(&cidr, ipstr, sizeof(ipstr))));
 
-	if ((dat->ipaddr.s_addr & netmask) == (arg->v.ip.s_addr & netmask)) 
+	if (mu_cidr_match(&cidr, &dat->addr) == 0)
 		return spf_term_match;
 	return spf_term_nomatch;
 }
 
-/* Not yet implemented: 
-     IP6              = "ip6"      ":" ip6-network   [ ip6-cidr-length ]
-*/
-static spf_term_result  
-mech_ip6(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
-{
-	mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
-		 ("ip6 mechanism is not yet supported"));
-	dat->result = spf_perm_error;
-	return spf_term_exception;
-}
-
 /* 5.7.
       exists           = "exists"   ":" domain-spec
 */
-static spf_term_result  
+static spf_term_result
 mech_exists(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
 	struct dns_reply r;
@@ -786,17 +861,21 @@ mech_exists(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 	}
 	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1,
 		 ("EXISTS domain_spec=%s", arg->v.domain_spec));
-	
+
+	/*
+	 * The resulting domain name is used for a DNS A RR lookup
+	 * (even when the connection type is IPv6).
+	 */
 	DNS_CATCH(a_lookup(arg->v.domain_spec, &r));
 	dns_reply_free(&r);
-	
+
 	return spf_term_match;
 }
 
 /* 6.1.
       redirect         = "redirect" "=" domain-spec
 */
-static spf_term_result  
+static spf_term_result
 mod_redirect(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
 	spf_query_t query;
@@ -804,7 +883,7 @@ mod_redirect(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 	query = dat->q;
 	query.domain = arg->v.domain_spec;
 	dat->result = spf_check_host_internal(&query, dat->answer, dat->loopno);
-	if (dat->result == spf_none) 
+	if (dat->result == spf_none)
 		dat->result = spf_perm_error;
 	return spf_term_exception;
 }
@@ -812,16 +891,16 @@ mod_redirect(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 /* 6.2.
       explanation      = "exp" "=" domain-spec
 */
-static spf_term_result  
+static spf_term_result
 mod_exp(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 {
 	struct dns_reply r;
-	
+
 	if (arg->v.domain_spec
 	    && txt_lookup(arg->v.domain_spec, &r) == dns_success) {
 		int i;
 		char *text;
-		
+
 		for (i = 0; i < r.count; i++) {
 			mu_opool_appendz(dat->tmpool, r.data.str[i]);
 		}
@@ -832,11 +911,11 @@ mod_exp(spf_data *dat, spf_term_arg *arg, unsigned long masklen)
 		if (dat->q.exp_prefix)
 			mu_opool_appendz(dat->tmpool, dat->q.exp_prefix);
 		if (spf_macro_expand(text, dat, 1)) {
-			mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR, 
+			mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
 				 ("error expanding explanation text %s",
 				  text));
 //FIXME			mu_opool_append_char(dat->tmpool, 0);
-		} else 
+		} else
 			dat->answer->exp_text = mu_strdup(spf_data_ptr(dat));
 	}
 	return spf_term_nomatch;
@@ -851,8 +930,8 @@ static struct spf_term_syntax term_syntax[] = {
 	{ "a", spf_term_mechanism, spf_arg_domain_spec, 32, mech_a },
 	{ "mx", spf_term_mechanism, spf_arg_domain_spec, 32, mech_mx },
 	{ "ptr", spf_term_mechanism, spf_arg_domain_spec, 0, mech_ptr },
-	{ "ip4", spf_term_mechanism, spf_arg_ipv4, 32, mech_ip4 },
-	{ "ip6", spf_term_mechanism, spf_arg_ipv6, 128, mech_ip6 },
+	{ "ip4", spf_term_mechanism, spf_arg_ipv4, 32, mech_ip },
+	{ "ip6", spf_term_mechanism, spf_arg_ipv6, 128, mech_ip },
 	{ "exists", spf_term_mechanism, spf_arg_domain_spec, 0, mech_exists },
 	{ "redirect", spf_term_modifier, spf_arg_domain_spec, 0,
 	  mod_redirect, SPF_MOD_REDIRECT },
@@ -892,20 +971,20 @@ parse_record(char *rec, struct spf_data *dat, int *ptermc, spf_term **ptermv)
 	spf_term modv[MAX_SPF_MOD];
 
 	memset(modv, 0, sizeof modv);
-	
+
 	/* Skip the initial version declaration */
 	skip_word(rec);
 
 	/* Allocate enough memory for debugging strings */
 	mu_opool_alloc(dat->tmpool, strlen(rec) + 1);
 	expr_space = mu_opool_finish(dat->tmpool, NULL);
-		
+
 	while (*rec) {
 		char *arg;
 		char *maskp = NULL;
 		spf_term_type type = spf_term_mechanism;
 		const struct spf_term_syntax *synt;
-		
+
 		skip_space(rec);
 		p = rec;
 		skip_word(rec);
@@ -915,23 +994,23 @@ parse_record(char *rec, struct spf_data *dat, int *ptermc, spf_term **ptermv)
 		term.expr = expr_space;
 		strcpy(expr_space, p);
 		expr_space += rec - p;
-		
+
 		switch (*p) {
 		case '+':
 			term.qualifier = spf_pass;
 			p++;
 			break;
-			
+
 		case '-':
 			term.qualifier = spf_fail;
 			p++;
 			break;
-			
+
 		case '~':
 			term.qualifier = spf_soft_fail;
 			p++;
 			break;
-			
+
 		case '?':
 			term.qualifier = spf_neutral;
 			p++;
@@ -941,7 +1020,7 @@ parse_record(char *rec, struct spf_data *dat, int *ptermc, spf_term **ptermv)
 			term.qualifier = spf_pass;
 		}
 
-		
+
 		for (arg = p; *arg; arg++)
 			if (*arg == ':') {
 				*arg++ = 0;
@@ -961,22 +1040,22 @@ parse_record(char *rec, struct spf_data *dat, int *ptermc, spf_term **ptermv)
 		synt = find_syntax(p);
 		if (!synt) {
 			if (type == spf_term_modifier) {
-				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR, 
+				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
 					 ("ignoring unknown modifier %s", p));
 				continue;
 			} else {
-				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR, 
+				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
 					 ("unknown mechanism %s", p));
 				return 1;
 			}
 		}
-		
+
 		if (synt->type != type) {
 			mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
 				 ("invalid use of %s", p));
 			return 1;
 		}
-		
+
 		term.type = synt->type;
 		term.handler = synt->handler;
 		term.masklen = synt->default_masklen;
@@ -1005,49 +1084,52 @@ parse_record(char *rec, struct spf_data *dat, int *ptermc, spf_term **ptermv)
 					return 1;
 				}
 			} else {
-				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR, 
-                                      ("error: netmask used with %s", p));
+				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
+				      ("error: netmask used with %s", p));
 				return 1;
 			}
 		}
-		
+
 		if (!*arg)
 			term.has_arg = 0;
 		else if (synt->argtype == spf_arg_none) {
-			mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR, 
-                              ("%s used with an argument", p));
+			mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
+			      ("%s used with an argument", p));
 			return 1;
 		} else {
 			term.has_arg = 1;
 			term.arg.type = synt->argtype;
 			switch (synt->argtype) {
 			default:
-				abort(); 
+				abort();
 
 			case spf_arg_domain_spec:
 				term.arg.v.domain_spec = arg;
 				break;
-				
+
 			case spf_arg_ipv4:
-				if (!inet_aton(arg, &term.arg.v.ip)) {
+				if (mu_cidr_from_string(&term.arg.v.ip, arg)
+				    || term.arg.v.ip.family != AF_INET) {
 					mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
 						 ("invalid IPv4: %s", arg));
 					return 1;
 				}
-				term.arg.v.ip.s_addr = ntohl(term.arg.v.ip.s_addr);
 				break;
-					  
+
 			case spf_arg_ipv6:
-				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR, 
-					 ("ignoring unsupported IPv6: %s",
-					  arg));
-				continue;
+				if (mu_cidr_from_string(&term.arg.v.ip, arg)
+				    || term.arg.v.ip.family != AF_INET6) {
+					mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
+						 ("invalid IPv6: %s", arg));
+					return 1;
+				}
+				break;
 			}
 		}
-		
+
 		if (term.type == spf_term_modifier) {
 			if (modv[synt->mod_index].handler) {
-				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR, 
+				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
 					 ("duplicate %s modifier", p));
 				return 1;
 			}
@@ -1060,7 +1142,7 @@ parse_record(char *rec, struct spf_data *dat, int *ptermc, spf_term **ptermv)
 
 	*ptermc = termc;
 	*ptermv = (spf_term*) spf_data_ptr(dat);
-	
+
 	for (i = 0; i < NELEMS(modv); i++)
 		dat->mod[i] = mu_opool_dup(dat->tmpool, &modv[i],
 					   sizeof modv[0]);
@@ -1074,7 +1156,7 @@ expand_term_arg(struct spf_data *dat, spf_term *term, spf_term_arg **parg)
 		spf_term_arg *arg = &term->arg;
 		if (arg->type == spf_arg_domain_spec) {
 			if (spf_macro_expand(arg->v.domain_spec, dat, 0)) {
-				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR, 
+				mu_debug(MF_SOURCE_SPF, MU_DEBUG_ERROR,
 					 ("error expanding %s",
 					  arg->v.domain_spec));
 				return 1;
@@ -1097,7 +1179,7 @@ spf_eval_record(char *rec, struct spf_data *dat)
 	spf_result result = spf_neutral;
 	const char *match = NULL;
 	spf_term_arg *parg;
-		
+
 	mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE0,
 		 ("Parsing SPF record: %s", rec));
 	if (parse_record(rec, dat, &termc, &termv))
@@ -1107,12 +1189,12 @@ spf_eval_record(char *rec, struct spf_data *dat)
 		 ("Evaluating SPF record"));
 	for (i = 0; i < termc; i++) {
 		tp = &termv[i];
-		
+
 		if (expand_term_arg(dat, tp, &parg)) {
 			result = spf_perm_error;
 			break;
 		}
-		    
+
 		mu_debug(MF_SOURCE_SPF, MU_DEBUG_TRACE1,
 			 ("SPF TERM: %s", tp->expr));
 		switch (tp->handler(dat, parg, tp->masklen)) {
@@ -1140,14 +1222,14 @@ spf_eval_record(char *rec, struct spf_data *dat)
 
 	if (match)
 		spf_answer_add_mech(dat->answer, match);
-	
+
 	if (!match && (tp = dat->mod[SPF_MOD_REDIRECT])->handler) {
-		if (expand_term_arg(dat, tp, &parg)) 
+		if (expand_term_arg(dat, tp, &parg))
 			result = spf_perm_error;
 		else {
 			spf_answer_add_mech(dat->answer, tp->expr);
 			if (tp->handler(dat, parg, tp->masklen)
-			    == spf_term_exception) 
+			    == spf_term_exception)
 				result = dat->result;
 			else if (dat->answer)
 				free(dat->answer->mechv[--dat->answer->mechn]);
@@ -1185,7 +1267,7 @@ spf_test_record(const char *rec, spf_query_t *q, spf_answer_t *a)
 {
 	struct spf_data dat;
 	spf_result result;
-	
+
 	mu_debug(MF_SOURCE_SPF,
 		 MU_DEBUG_TRACE0,
 		 ("SPF record: %s, ip=%s, domain=%s, sender=%s",
@@ -1205,7 +1287,7 @@ spf_test_record(const char *rec, spf_query_t *q, spf_answer_t *a)
 		 ("result = %s", spf_result_str[result]));
 
 	return result;
-}	
+}
 
 char *
 spf_test_macro_expand(const char *input, spf_query_t *q, int allow_exp)
@@ -1248,10 +1330,10 @@ spf_check_host_internal(spf_query_t *q, spf_answer_t *a, size_t loopno)
 			 ("SPF recursion limit reached"));
 		return spf_perm_error;
 	}
-	
+
 	if (a)
 		memset(a, 0, sizeof *a);
-	
+
 	switch (spf_lookup(q->domain, &spf_rec)) {
 	case dns_success:
 		break;
@@ -1259,7 +1341,7 @@ spf_check_host_internal(spf_query_t *q, spf_answer_t *a, size_t loopno)
 	case dns_too_many:
 		SPF_RETURN(spf_perm_error,
 			   "too many SPF records published");
-		
+
 	case dns_not_found:
 	case dns_failure:
 		SPF_RETURN(spf_none,
@@ -1274,9 +1356,9 @@ spf_check_host_internal(spf_query_t *q, spf_answer_t *a, size_t loopno)
 
 	if (spf_data_init(&dat, q, a, loopno + 1))
 		SPF_RETURN(spf_perm_error, "spf_data_init failed");
-	
+
 	result = spf_exec_query(spf_rec, &dat);
-	
+
 	spf_data_free(&dat);
 	free(spf_rec);
 
@@ -1293,7 +1375,7 @@ void
 spf_answer_free(spf_answer_t *ans)
 {
 	size_t i;
-	
+
 	free(ans->exp_text);
 	for (i = 0; i < ans->mechn; i++)
 		free(ans->mechv[i]);
@@ -1314,4 +1396,3 @@ spf_answer_add_mech(spf_answer_t *ans, char const *mech)
 	}
 	ans->mechv[ans->mechn++] = mu_strdup(mech);
 }
-		
